package com.sun.org.apache.xalan.internal.xsltc.compiler;

import com.sun.org.apache.bcel.internal.generic.ConstantPoolGen;
import com.sun.org.apache.bcel.internal.generic.PUSH;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.MethodGenerator;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.ClassGenerator;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.Util;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import com.sun.org.apache.xalan.internal.utils.ObjectFactory;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.TypeCheckError;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.ErrorMsg;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.Type;
import java.util.List;

final class FunctionAvailableCall extends FunctionCall
{
    private Expression _arg;
    private String _nameOfFunct;
    private String _namespaceOfFunct;
    private boolean _isFunctionAvailable;
    
    public FunctionAvailableCall(QName fname, List<Expression> arguments) {
        super(fname, arguments);
        this._nameOfFunct = null;
        this._namespaceOfFunct = null;
        this._isFunctionAvailable = false;
        this._arg = (Expression)arguments.get(0);
        this._type = null;
        if ((this._arg instanceof LiteralExpr)) {
            LiteralExpr arg = (LiteralExpr)this._arg;
            this._namespaceOfFunct = arg.getNamespace();
            this._nameOfFunct = arg.getValue();
            if (!this.isInternalNamespace()) {
                this._isFunctionAvailable = this.hasMethods();
            }
        }
    }
    
    @Override
    public Type typeCheck(SymbolTable stable) throws TypeCheckError {
        if (this._type != null) {
            return this._type;
        }
        if ((this._arg instanceof LiteralExpr)) {
            return this._type = Type.Boolean;
        }
        ErrorMsg err = new ErrorMsg("NEED_LITERAL_ERR", "function-available", this);
        throw new TypeCheckError(err);
    }
    
    @Override
    public Object evaluateAtCompileTime() {
        return this.getResult() ? Boolean.TRUE : Boolean.FALSE;
    }
    
    private boolean hasMethods() {
        String className = this.getClassNameFromUri(this._namespaceOfFunct);
        String methodName = null;
        int colonIndex = this._nameOfFunct.indexOf(":");
        if (colonIndex > 0) {
            String functionName = this._nameOfFunct.substring(colonIndex + 1);
            int lastDotIndex = functionName.lastIndexOf(46);
            if (lastDotIndex > 0) {
                methodName = functionName.substring(lastDotIndex + 1);
                if (className != null && className.length() != 0) {
                    className = className + "." + functionName.substring(0, lastDotIndex);
                }
                else {
                    className = functionName.substring(0, lastDotIndex);
                }
            }
            else {
                methodName = functionName;
            }
        }
        else {
            methodName = this._nameOfFunct;
        }
        if (className == null || methodName == null) {
            return false;
        }
        if (methodName.indexOf(45) > 0) {
            methodName = FunctionCall.replaceDash(methodName);
        }
        try {
            Class<?> clazz = ObjectFactory.findProviderClass(className, true);
            if (clazz == null) {
                return false;
            }
            Method[] methods = clazz.getMethods();
            int i = 0;
            while (i < methods.length) {
                int mods = methods[i].getModifiers();
                if (Modifier.isPublic(mods) && Modifier.isStatic(mods) && methods[i].getName().equals(methodName)) {
                    return true;
                }
                ++i;
            }
        }
        catch (ClassNotFoundException e) {
            return false;
        }
        return false;
    }
    
    public boolean getResult() {
        if (this._nameOfFunct == null) {
            return false;
        }
        if (this.isInternalNamespace()) {
            Parser parser = this.getParser();
            this._isFunctionAvailable = parser.functionSupported(Util.getLocalName(this._nameOfFunct));
        }
        return this._isFunctionAvailable;
    }
    
    private boolean isInternalNamespace() {
        return this._namespaceOfFunct == null || this._namespaceOfFunct.equals("") || this._namespaceOfFunct.equals("http://xml.apache.org/xalan/xsltc");
    }
    
    @Override
    public void translate(ClassGenerator classGen, MethodGenerator methodGen) {
        ConstantPoolGen cpg = classGen.getConstantPool();
        methodGen.getInstructionList().append(new PUSH(cpg, this.getResult()));
    }
}
